<?php
/********************************
 * File : uspsTools.php
 *
 * Description : Implements a class to access the USPS online tools.
 * This inital version supports only the USPS rate lookups.  Future
 * versions will add the address checking as well.
 *
 * Class Functions:
 *    uspsTools(usps_customer_id, originating_zip) - Constructor
 *    GetRateUS(dest_zip, weight_lbs) - Get all services and rates
 *    GetRateUS_byService(dest_zip, weight_lbs, service) Get single rate
 *    GetRateIntl(dest_country, weight_lbs, value) Get all services and rates
 *    GetRateIntl_byService(dest_country, weight_lbs, value) Get single rate
 *    GetServicesUS(dest_zip, weight) - TODO
 *    GetServicesIntl(dest_country, weight, value) - TODO
 *
 *******************************/


/****
 * Example
 *

$usps = new uspsTools('usps_cust_id', '10101');
$xml = $usps->GetRateUS('33511', 2);

// print_r($xml);
foreach($xml->Package->Postage as $postage) {
  // print_r($postage);
  if (($postage->MailService == 'Priority Mail') ||
      ($postage->MailService == 'Parcel Post')) {
    print $postage->MailService . " : " . $postage->Rate . "<br>\n";
  }
}
$rate = $usps->GetRateUS_byService('33511', 12, 'Parcel Post');
print "Parcel Post : " . $rate . "<br>\n";
$rate = $usps->GetRateUS_byService('33511', 12, 'Priority Mail');
print "Priority Mail : " . $rate . "<br>\n";
$rate = $usps->GetRateUS_byService('84114', 21, 'Priority Mail');
print "Priority Mail : " . $rate . "<br>\n";
$rate = $usps->GetRateUS_byService('33511', 2, 'Parcel Post');
print "Parcel Post : " . $rate . "<br>\n";
print "<p>\n";

// print "<pre>\n";
$xml= $usps->GetRateIntl('Canada', 12, 150);
print_r($xml);
foreach($xml->Package->Service as $postage) {
  if (($postage->SvcDescription == 'Priority Mail International') ||
      ($postage->SvcDescription == 'Global Express Guaranteed')) {
    print $postage->SvcDescription . ' : ' . $postage->Postage . "<br>\n";
  }
  // print_r($postage);
}
// print "</pre>\n";

$rate = $usps->GetRateIntl_byService('Canada', 12, 150, 'Priority Mail International');
print "Priority Mail International : " . $rate . "<br>\n";
$rate = $usps->GetRateIntl_byService('Canada', 12, 150, 'Global Express Guaranteed');
print "Global Express Guaranteed : " . $rate . "<br>\n";

 * END OF EXAMPLE
 ******************/

include_once('class_http.php');

class uspsTools {

  function uspsTools($userid, $originatingZIP) {
    // $this->baseURL = 'http://testing.ShippingAPIs.com/ShippingAPI.dll';
    $this->baseURL = 'http://Production.ShippingAPIs.com/ShippingAPI.dll';
    $this->userid = $userid;
    $this->originatingZIP = $originatingZIP;
    $this->curr_destZIP = '';
    $this->curr_destCountry = '';
    $this->curr_ShipWt = 0;
    $this->cur_value = 0;
    $this->curr_XML = '';
  }

  function GetRateUS($dest_zip, $weight) {
    $xmlstr = <<<XML
<RateV3Request>
  <Package ID="0">
    <Service>ALL</Service>
    <ZipOrigination>84321</ZipOrigination>
    <ZipDestination></ZipDestination>
    <Pounds></Pounds>
    <Ounces>0</Ounces>
    <Container></Container>
    <Size>REGULAR</Size>
    <Machinable>true</Machinable>
  </Package>
</RateV3Request>
XML;

    $this->curr_destZIP = $dest_zip;
    $this->curr_ShipWt = $weight;
    $this->curr_destCountry = '';
    $xml = new SimpleXMLElement($xmlstr);
    $h = new http();

    $xml->addAttribute('USERID', $this->userid);
    $xml->Package->ZipDestination = $dest_zip;
    $xml->Package->Pounds = $weight;
 
    // Prepare for XML call
    $h->xmlrequest = $xml->asXML();
 
    $output = str_replace(array("\n",'<?xml version="1.0"?>',"\t",),array('','',''),$xml->asXML());
    $url = $this->baseURL . '?API=RateV3&XML=' . urlencode($output);
 
    // Submit data and get results
    $h->fetch($url,0,'','','','GET');
 
    // print_r($h->body);die();
 
    // Put results into XML object
    $xml = new SimpleXMLElement($h->body);
 
    $this->curr_XML = $xml; 
    return $xml;
  }

  function GetRateIntl($dest_country, $weight, $value) {
    $xmlstr = <<<XML
<IntlRateRequest>
  <Package ID="0">
    <Pounds></Pounds>
    <Ounces>0</Ounces>
    <Machinable>true</Machinable>
    <MailType>Package</MailType>
    <ValueOfContents></ValueOfContents>
    <Country></Country>
  </Package>
</IntlRateRequest>
XML;

    $this->curr_destZIP = '';
    $this->curr_ShipWt = $weight;
    $this->curr_destCountry = $dest_country;
    $this->cur_value = $value;
    $xml = new SimpleXMLElement($xmlstr);
    $h = new http();

    $xml->addAttribute('USERID', $this->userid);
    $xml->Package->Country = $dest_country;
    $xml->Package->Pounds = $weight;
    $xml->Package->ValueOfContents = $value;
 
    // Prepare for XML call
    $h->xmlrequest = $xml->asXML();
 
    $output = str_replace(array("\n",'<?xml version="1.0"?>',"\t",),array('','',''),$xml->asXML());
    // print_r($output);
    $url = $this->baseURL . '?API=IntlRate&XML=' . urlencode($output);
 
    // Submit data and get results
    $h->fetch($url,0,'','','','GET');
 
    // print_r($h->body);die();
 
    // Put results into XML object
    $xml = new SimpleXMLElement($h->body);

    $this->curr_XML = $xml; 
    return $xml;
  }

  function GetRateUS_byService($zip, $weight, $service) {
    if (($zip != $this->curr_destZIP) ||
	($weight != $this->curr_ShipWt)) {
      $this->GetRateUS($zip, $weight);
    }
    $xml = $this->curr_XML;
    foreach($xml->Package->Postage as $postage) {
      if ($postage->MailService == $service) {
	return $postage->Rate;
      }
    }
    return 0;
  }

  function GetRateIntl_byService($country, $weight, $value, $service) {
    if (($country != $this->curr_destCountry) ||
	($weight != $this->curr_ShipWt) ||
	($value != $this->curr_value)) {
      $this->GetRateIntl($country, $weight, $value);
    }
    $xml = $this->curr_XML;
    foreach($xml->Package->Service as $postage) {
      if ($postage->SvcDescription == $service) {
	return $postage->Postage;
      }
    }
    return 0;
  }

  function GetServicesUS($zip, $weight) {
    // TODO
    // return a list of the services available for this zip code and package weight
  }

  function GetServicesIntl($country, $weight, $value) {
    // TODO
    // return a list of the services available for this country, weight and value
  }
}

?>